package Test::Catalyst::Action::REST;

use strict;
use warnings;

use Catalyst::Runtime '5.70';

use Catalyst;
use FindBin;

__PACKAGE__->config(
    name => 'Test::Catalyst::Action::REST',
    # RT#43840 -- this was ignored in 0.66 and earlier
    'Controller::Serialize' => {
        content_type_stash_key => 'serialize_content_type',
    },
);
__PACKAGE__->setup;

1;
